@@ -80,6 +80,12 @@ struct RhythmDBPrivate
 
 	GMutex *exit_mutex;
 	gboolean exiting;
+	
+	GCond *saving_condition;
+	GMutex *saving_mutex;
+
+	gboolean saving;
+	gboolean dirty;
 };
 
 struct RhythmDBAction
@@ -143,6 +149,7 @@ enum
 	ENTRY_CHANGED,
 	ENTRY_DELETED,
 	LOAD_COMPLETE,
+	SAVE_COMPLETE,
 	ERROR,
 	LAST_SIGNAL
 };
@@ -263,6 +270,17 @@ rhythmdb_class_init (RhythmDBClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	rhythmdb_signals[SAVE_COMPLETE] =
+		g_signal_new ("save_complete",
+			      RHYTHMDB_TYPE,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RhythmDBClass, save_complete),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 	rhythmdb_signals[ERROR] =
 		g_signal_new ("error",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -469,6 +487,13 @@ rhythmdb_init (RhythmDB *db)
 	g_async_queue_ref (db->priv->main_thread_cancel);
 	g_async_queue_ref (db->priv->action_queue);
 	g_thread_create ((GThreadFunc) action_thread_main, db, FALSE, NULL);
+	
+	db->priv->saving_condition = g_cond_new ();
+	db->priv->saving_mutex = g_mutex_new ();
+
+	db->priv->exiting = FALSE;
+	db->priv->saving = FALSE;
+	db->priv->dirty = FALSE;
 }
 
 gboolean
@@ -531,6 +556,9 @@ rhythmdb_finalize (GObject *object)
 
 	g_async_queue_unref (db->priv->status_queue);
 
+	g_mutex_free (db->priv->saving_mutex);
+	g_cond_free (db->priv->saving_condition);
+
 	g_free (db->priv->column_types);
 
 	g_hash_table_destroy (db->priv->propname_map);
@@ -609,32 +637,35 @@ rhythmdb_write_lock (RhythmDB *db)
 {
 	g_static_rw_lock_writer_lock (&db->priv->lock);
 
-	db->priv->added_entries = NULL;
-	db->priv->changed_entries = NULL;
+	g_assert (db->priv->added_entries == NULL);
+	g_assert (db->priv->changed_entries == NULL);
 }
 
 static void
-emit_changed_signals (RhythmDB *db)
+emit_changed_signals (RhythmDB *db, gboolean commit)
 {
 	GList *tem;
 	GHashTable *queued_entry_changes = g_hash_table_new (NULL, NULL);
 
 	for (tem = db->priv->changed_entries; tem; tem = tem->next) {
 		struct RhythmDBEntryChangeData *data = tem->data;
-		struct RhythmDBAction *action;
-		RBMetaDataField field;
 
-		if (rhythmdb_entry_get_int (db, data->entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_SONG
-		    && !g_hash_table_lookup (queued_entry_changes, data->entry)
-		    && metadata_field_from_prop (data->prop, &field)) {
-			action = g_new0 (struct RhythmDBAction, 1);
-			action->type = RHYTHMDB_ACTION_SYNC;
-			action->entry = data->entry;
-			g_hash_table_insert (queued_entry_changes, data->entry, action);
-			g_async_queue_push (db->priv->action_queue, action);
+		if (commit) {
+			struct RhythmDBAction *action;
+			RBMetaDataField field;
+
+			if (rhythmdb_entry_get_int (db, data->entry, RHYTHMDB_PROP_TYPE) == RHYTHMDB_ENTRY_TYPE_SONG
+			    && !g_hash_table_lookup (queued_entry_changes, data->entry)
+			    && metadata_field_from_prop (data->prop, &field)) {
+				action = g_new0 (struct RhythmDBAction, 1);
+				action->type = RHYTHMDB_ACTION_SYNC;
+				action->entry = data->entry;
+				g_hash_table_insert (queued_entry_changes, data->entry, action);
+				g_async_queue_push (db->priv->action_queue, action);
+			}
+			g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_CHANGED], 0, data->entry,
+				       data->prop, &data->old, &data->new);
 		}
-		g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_CHANGED], 0, data->entry,
-			       data->prop, &data->old, &data->new);
 		g_value_unset (&data->old);
 		g_value_unset (&data->new);
 		g_free (data);
@@ -647,10 +678,9 @@ emit_changed_signals (RhythmDB *db)
 static void
 rhythmdb_write_unlock_internal (RhythmDB *db, gboolean commit)
 {
-	GList *tem;
+       GList *tem;
 
-	if (commit)
-		emit_changed_signals (db);
+	emit_changed_signals (db, commit);
 
 	for (tem = db->priv->added_entries; tem; tem = tem->next)
 		g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_ADDED], 0, tem->data);
@@ -743,7 +773,7 @@ set_metadata_string_default_unknown (RhythmDB *db, RhythmDBEntry *entry,
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_static_string (&val, unknown);
 	} else if (g_value_get_string (&val)[0] == '\0')
-		g_value_set_string (&val, unknown);
+		g_value_set_static_string (&val, unknown);
 	rhythmdb_entry_set (db, entry, prop, &val);
 	g_value_unset (&val);
 }
@@ -877,8 +907,9 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry)
 RhythmDBEntry *
 rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 {
-	RhythmDBEntry *entry;
+	RhythmDBEntry *entry = NULL;
 	char *realuri;
+	const char *mime;
 	GError *error = NULL;
 	GValue value = {0, };
 
@@ -905,8 +936,14 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 	if (error) {
 		g_propagate_error (real_error, error);
 		rb_debug ("failed to read data from \"%s\"", uri);
-		g_free (realuri);
-		return NULL;
+		goto out_freeuri;
+	}
+
+	mime = rb_metadata_get_mime (db->priv->metadata);
+
+	if (!mime) {
+		rb_debug ("unsupported file");
+		goto out_freeuri;
 	}
 
 	rhythmdb_write_lock (db);
@@ -933,11 +970,12 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **real_error)
 	g_value_set_boolean (&value, TRUE);
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_AUTO_RATE, &value);
 	g_value_unset (&value);
-
+	
  out_dupentry:
+	rhythmdb_write_unlock_internal (db, FALSE);
+ out_freeuri:
 	g_free (realuri);
 
-	rhythmdb_write_unlock_internal (db, FALSE);
 	return entry;
 }
 
@@ -1024,7 +1062,7 @@ add_thread_main (RhythmDB *db)
 		else
 			rb_uri_handle_recursively (uri, (GFunc) add_file,
 						   &db->priv->exiting, db);
-
+						 
 		if (error != NULL)
 			push_err (db, realuri, error);
 
@@ -1195,6 +1233,7 @@ rhythmdb_load_thread_main (RhythmDB *db)
 
 	g_signal_emit (G_OBJECT (db), rhythmdb_signals[LOAD_COMPLETE], 0);
 	g_async_queue_push (db->priv->status_queue, db);
+	
 	return NULL;
 }
 
@@ -1203,17 +1242,85 @@ rhythmdb_load (RhythmDB *db)
 {
 	g_object_ref (G_OBJECT (db));
 	db->priv->outstanding_threads++;
-	g_thread_create ((GThreadFunc) rhythmdb_load_thread_main, db, TRUE, NULL);
+	g_thread_create ((GThreadFunc) rhythmdb_load_thread_main, db, FALSE, NULL);
+}
+
+static void
+rhythmdb_save_worker (RhythmDB *db)
+{
+	RhythmDBClass *klass;
+	
+	g_mutex_lock (db->priv->saving_mutex);
+
+	if (db->priv->saving) {
+		rb_debug ("already saving, ignoring");
+		g_mutex_unlock (db->priv->saving_mutex);
+		return;
+	}
+
+	if (!db->priv->dirty) {
+		rb_debug ("no save needed, ignoring");
+		g_mutex_unlock (db->priv->saving_mutex);
+		return;
+	}
+
+	db->priv->saving = TRUE;
+
+	g_mutex_unlock (db->priv->saving_mutex);
+
+	rb_debug ("saving rhythmdb");
+			
+	klass = RHYTHMDB_GET_CLASS (db);
+	db_enter (db, FALSE);
+	klass->impl_save (db);
+
+	g_mutex_lock (db->priv->saving_mutex);
+
+	db->priv->saving = FALSE;
+	db->priv->dirty = FALSE;
+
+	g_mutex_unlock (db->priv->saving_mutex);
+
+	g_cond_broadcast (db->priv->saving_condition);
+}
+
+static gpointer
+rhythmdb_save_thread_main (RhythmDB *db)
+{
+	rb_debug ("entering save thread");
+	
+	rhythmdb_save_worker (db);
+
+	g_signal_emit (G_OBJECT (db), rhythmdb_signals[SAVE_COMPLETE], 0);
+	g_async_queue_push (db->priv->status_queue, db);
+	
+	return NULL;
 }
 
 void
 rhythmdb_save (RhythmDB *db)
 {
-	RhythmDBClass *klass = RHYTHMDB_GET_CLASS (db);
+	rb_debug ("saving the rhythmdb in the background");
 
-	db_enter (db, FALSE);
+	g_object_ref (G_OBJECT (db));
+	db->priv->outstanding_threads++;
 	
-	klass->impl_save (db);
+	g_thread_create ((GThreadFunc) rhythmdb_save_thread_main, db, FALSE, NULL);
+}
+
+void
+rhythmdb_save_blocking (RhythmDB *db)
+{
+	rb_debug("saving the rhythmdb and blocking");
+	
+	rhythmdb_save_worker (db);
+	
+	g_mutex_lock (db->priv->saving_mutex);
+
+	while (db->priv->saving)
+		g_cond_wait (db->priv->saving_condition, db->priv->saving_mutex);
+
+	g_mutex_unlock (db->priv->saving_mutex);
 }
 
 static void
@@ -1293,6 +1400,9 @@ rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry,
 	klass->impl_entry_set (db, entry, propid, value);
 
 	rhythmdb_entry_sync_mirrored (db, entry, propid, value);
+	
+	/* set the dirty state */
+	db->priv->dirty = TRUE;
 }
 
 void
@@ -1385,6 +1495,9 @@ rhythmdb_entry_delete (RhythmDB *db, RhythmDBEntry *entry)
 	rhythmdb_emit_entry_deleted (db, entry);
 	
 	klass->impl_entry_delete (db, entry);
+	
+	/* deleting an entry makes the db dirty */
+	db->priv->dirty = TRUE;
 }
 
 void
